# Designed May 2010 by ChrisP

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is forbidden:

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name            :  ChrisP_ColorEdge
# Original Date   :  14 May 2010 version 0.0
# Thanks          :  Jim Foltz for clinetools, TIG for ListLayerColors
# Type            :  Plugin
# Description     :  Edge colored, Dotted lines Colored, Angle Colored, Colored by layer
# Usage           :  By ToolBar
# Version         :  0.2.3
#-------------------------------------------------------------------------------------------------------------------------------------------
require 'sketchup.rb'

class Float
  def round_to(x)
   (self*10**x).round.to_f/10**x
  end 
end

class Calque_Observeur < Sketchup::LayersObserver 
       def onLayerAdded(layers, layer)
          calque_export           
       end      
end
      
def calque_export
	  model=Sketchup.active_model
	  entities = model.active_entities    
	  materials=model.materials
	  layers=model.layers
	  rendering_options = model.rendering_options
	  mat_defaut = model.active_layer
	  if rendering_options["DisplayColorByLayer"]==false
	     rendering_options["DisplayColorByLayer"]=true
		already_on=false
	  else
	    already_on=true
	  end#if
	  # create temporary entities for exporting layers
	  vecteur = Geom::Vector3d.new 0,0,1
	  centre = Geom::Point3d.new 0,0,0
	  group_temp = entities.add_group()
	  entities = group_temp.entities
	  x =0
	  layers.each {|layer|
	  model.active_layer = layer
	  centre = [centre[0], centre[1], centre[2] + x]
	  edges = entities.add_circle centre, vecteur, 0.01
	  cercle = entities.add_face(edges)
	  x= x + 0.01
	  }
	   
  	  mpath = Sketchup.find_support_file "Sketchup.exe"
  	  mpath.gsub!(/Sketchup.exe/,'temp')
	  model.export(mpath + ".obj",false)
	  mtl = mpath + ".mtl"
	  text = IO.readlines(mtl)
	 0.upto(text.length-1) do |i|
	  if text[i]=~/^newmtl */
	    name=text[i].slice(7..-2)### layer name only
	    rgbR=((text[i+2].slice(3..11).to_f)*255).to_i.to_s### color rgb values	
	    rgbG=((text[i+2].slice(12..20).to_f)*255).to_i.to_s### color rgb values	
	    rgbB=((text[i+2].slice(21..29).to_f)*255).to_i.to_s### color rgb values
	    
	  	couleur_calque = Sketchup::Color.new(rgbR.to_i, rgbG.to_i, rgbB.to_i)
	        unless materials[name]
		    new_matiere = materials.add name
		    new_matiere.color = couleur_calque
		    new_matiere = new_matiere.name
		else
		    materials[name].color = couleur_calque
		end
		
	  end#if
	 end#upto
	  group_temp.erase!
	  File.delete(mpath + ".obj")
	  File.delete(mtl)
	    
	  rendering_options["DisplayColorByLayer"]=false if already_on==false
	  model.active_layer = mat_defaut
	
end #Fin calque_export

class ColorEdge

def initialize(type_color = "single")

    @type_color = type_color
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
    @curseur = 0
    @model = Sketchup.active_model
    @entities = @model.entities
    @selection = @model.selection
    @materials = @model.materials
    @layers = @model.layers
    @rendering_options = @model.rendering_options
    @layers.add_observer @observeur1 = Calque_Observeur.new		
    value = @rendering_options["EdgeColorMode"]=0
    path = File.join(File.dirname(__FILE__),  "ChrisP_ColorEdge", "coloredge_cursor.png")
    @curseur = UI::create_cursor(path, 1, 30)
end
def onSetCursor

      UI::set_cursor(@curseur)
  
end
def activate(color1 = "Red", color2 = "Green", color3 = "Blue", color4 = "Yellow", color5 = "Magenta")
   (@materials.current) ?(@defaut_color = @materials.current.color) :(@defaut_color = nil)
   (@materials.current) ?(@defaut1_color = @materials.current.color) :(@defaut1_color = "black")
    
   (@type_color == "multi") ?(@color1 = color1) :(@color1 =  @defaut_color)    
   (@type_color == "multi") ?(@color2 = color2) :(@color2 =  @defaut_color)
   (@type_color == "multi") ?(@color3 = color3) :(@color3 =  @defaut_color)
   (@type_color == "multi") ?(@color4 = color4) :(@color4 =  @defaut_color)
   (@type_color == "multi") ?(@color5 = color5) :(@color5 =  @defaut_color)
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false

    Sketchup::set_status_text "Length", SB_VCB_LABEL
    if @rendering_options["HideConstructionGeometry"] == true 
       @rendering_options["HideConstructionGeometry"] = false 
    end
    self.reset(nil)
end

def resume(view)
        Sketchup::set_status_text "TAB to modify parameters", SB_PROMPT
end
    
def deactivate(view)
    view.invalidate if @drawn
    @layers.remove_observer @observeur1
    @materials.purge_unused 
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )

        @ip.pick view, x, y
        if( @ip != @ip1 )

            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            
            view.tooltip = @ip1.tooltip
        end
    else

        @ip2.pick view, x, y, @ip1
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        
        if( @ip2.valid? )
            length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
        end
        
        if((x-@xdown).abs > 10 ||(y-@ydown).abs > 10 )
            @dragging = true
        end
    end
    
end

def onLButtonDown(flags, x, y, view)
 
    if( @state == 0 )
        @ip1.pick view, x, y
        
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text "选择结束点，按TAB键可以设置不同方向的颜色 == TAB to modify colors", SB_PROMPT
            @xdown = x
            @ydown = y
        end
    else

        if( @ip2.valid? )
            self.create_geometry(@ip1.position, @ip2.position,view)
            self.reset(view)
        end
    end
    
    view.lock_inference
end

def onLButtonUp(flags, x, y, view)
    if( @dragging && @ip2.valid? )
        self.create_geometry(@ip1.position, @ip2.position,view)
        self.reset(view)
    end
end

def onKeyDown(key, repeat, flags, view)

    if key == 9
    	if(@type_color == "multi")
		table = (["Default"] + Sketchup::Color.names).join("|")
		prompts = ["Color on axis X", "Color on axis Y", "Color on axis Z", "Color 45 degree angle", "Color other axis"]    
		val_defaut = ["Red", "Green", "Blue", "Yellow", "Magenta"]
		liste_val = [table,table,table,table,table]
		results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
		return if not results
		colr1, colr2, colr3, colr4, colr5 = results
		@color1 = colr1.to_s
	        @color2 = colr2.to_s
	        @color3 = colr3.to_s
	        @color4 = colr4.to_s
	        @color5 = colr5.to_s
        elsif(@type_color == "single")
        	table = (["Default","Active Layer","Red", "Green", "Blue", "Yellow", "Magenta"] + Sketchup::Color.names).join("|")
		prompts = ["Color single"]   
		val_defaut = ["Magenta"]
		liste_val = [table]
		results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
		return if not results
		colr1 = results
		@calque = colr1.to_s		
		@color1 = colr1.to_s
		calque_export if(@calque == "Active Layer")
        end
    end
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        if( view.inference_locked? )
            view.lock_inference            
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
            view.line_width = 3
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
            view.line_width = 3
        end
    end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
       (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)
    
    begin
        value = text.to_l
    rescue
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    

    if @state == 1 and @ip2.valid?
            pt1 = @ip1.position
            vec = @ip2.position - pt1
            if( vec.length == 0.0 )
                UI.beep
                return
            end
            vec.length = value
            pt2 = pt1 + vec

            self.create_geometry(pt1, pt2, view)
            self.reset(view)
        end
        if @last_drawn and @state == 0
            pt1 = @last_drawn[0].start
            vec = @last_drawn[0].end - @last_drawn[0].start
            vec.length = value
            pt2 = pt1 + vec
            view.model.active_entities.erase_entities @last_drawn
            self.create_geometry(pt1, pt2, view)
            self.reset view
        end
end

def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry(@ip1.position, @ip2.position, view)
            @drawn = true
        end
    end
end

def onCancel(flag, view)
    self.reset(view)
end

def reset(view)
  
    @state = 0
    
    Sketchup::set_status_text("选择开始点，按TAB键可以设置不同方向的颜色", SB_PROMPT)
    
    @ip1.clear
    @ip2.clear
    @ip.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end
def create_geometry(p1, p2, view)

    @model.start_operation("ColorEdge")
    edge = view.model.entities.add_line(p1,p2)
    vecteur = p1.vector_to p2
    vecteur.x = -vecteur.x if(vecteur.x < 0)
    vecteur.y = -vecteur.y if(vecteur.y < 0)
    vecteur.z = -vecteur.z if(vecteur.z < 0)
    vecteur.normalize!
    vecteur.x = vecteur.x.round_to(7)
    vecteur.y = vecteur.y.round_to(7)
    vecteur.z = vecteur.z.round_to(7)
    @color1 = @materials[ @model.active_layer.name].color if(@calque == "Active Layer")    
        if(@type_color == "unique")
           @color2 = @color1 
           @color3 = @color1
           @color4 = @color1
           @color5 = @color1
        end     	
        if(vecteur.x == 1)
      		(@color1 == "Default") ?(edge.material = @defaut_color) :(edge.material = @color1)
      	elsif(vecteur.y == 1)
      		(@color2 == "Default") ?(edge.material = @defaut_color) :(edge.material = @color2)
      	elsif(vecteur.z == 1)
      		(@color3 == "Default") ?(edge.material = @defaut_color) :(edge.material = @color3)
  	elsif(((vecteur.x == vecteur.y) and(vecteur.x + vecteur.y != 0) ) or((vecteur.x == vecteur.z) and(vecteur.x + vecteur.z != 0) )or((vecteur.y == vecteur.z) and(vecteur.y + vecteur.z != 0) ))
      		(@color4 == "Default") ?(edge.material = @defaut_color) :(edge.material = @color4)
      	else
      		(@color5 == "Default") ?(edge.material = @defaut_color) :(edge.material = @color5)
      	end  
    @model.commit_operation
     
end

def draw_geometry(pt1, pt2, view)
    vecteur = pt1.vector_to pt2 
    vecteur.x = -vecteur.x if(vecteur.x < 0)
    vecteur.y = -vecteur.y if(vecteur.y < 0)
    vecteur.z = -vecteur.z if(vecteur.z < 0)
    vecteur.normalize!
    vecteur.x = vecteur.x.round_to(7)
    vecteur.y = vecteur.y.round_to(7)
    vecteur.z = vecteur.z.round_to(7)
    @color1 = @materials[ @model.active_layer.name].color if(@calque == "Active Layer")
    
        if(@type_color == "single")
          @color2 = @color1 
          @color3 = @color1
          @color4 = @color1
          @color5 = @color1
        end  
    	if(vecteur.x == 1)
    	        (@color1 == "Default" or @color1 == nil) ?(view.drawing_color = @defaut1_color) :(view.drawing_color = @color1)
    	elsif(vecteur.y == 1)
        	(@color2 == "Default" or @color2 == nil) ?(view.drawing_color = @defaut1_color) :(view.drawing_color = @color2)
    	elsif(vecteur.z == 1)
    		(@color3 == "Default" or @color3 == nil) ?(view.drawing_color =@defaut1_color) :(view.drawing_color = @color3)
    	elsif(((vecteur.x == vecteur.y) and(vecteur.x + vecteur.y != 0) ) or((vecteur.x == vecteur.z) and(vecteur.x + vecteur.z != 0) )or((vecteur.y == vecteur.z) and(vecteur.y + vecteur.z != 0) ))
    		(@color4 == "Default" or @color4 == nil) ?(view.drawing_color = @defaut1_color) :(view.drawing_color = @color4)
    	else
    		(@color5 == "Default" or @color5 == nil) ?(view.drawing_color = @defaut1_color) :(view.drawing_color = @color5)
    	end   	
    view.line_width = 3	 
    view.draw_line(pt1, pt2)	
end

def getExtents
        bbox = Sketchup.active_model.bounds
        bbox.add(@ip.position) if @ip and @ip.valid?
        bbox.add(@ip1.position)if @ip1 and @ip1.valid?
        bbox.add(@ip2.position)if @ip2 and @ip2.valid?

        return bbox
    end
end # class ColorEdge
def colorEdge_params

   def couleur
	calque=[]
	Sketchup.active_model.layers.each{|x|calque.push(x.name)}
	calque = ["All"] + calque
	calque=calque.join('|')
	table = (["Delete","Default"] + Sketchup::Color.names).join("|")
	prompts = ["通过图层","在x轴颜色", "在Y轴颜色", "在Z轴颜色", "45度方向的颜色", "其他方向颜色"]    
	val_defaut = ["All","Red", "Green", "Blue", "Yellow", "Magenta"]
	liste_val = [calque,table,table,table,table,table]
	results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
	@calque, @colr1, @colr2, @colr3, @colr4, @colr5 = results
	calque = @calque	
        color1 = @colr1
        color2 = @colr2
        color3 = @colr3
        color4 = @colr4
        color5 = @colr5
        return calque, color1, color2, color3, color4, color5
    end
end

def colorEdge_params1

   def couleur
	calque=[]
	Sketchup.active_model.layers.each{|x|calque.push(x.name)}
	calque = ["All"] + calque
	calque=calque.join('|')
	table = (["Delete","Default","Color Layer"] + Sketchup::Color.names).join("|")
	prompts = ["图层","单一颜色边线"]    
	val_defaut = ["All","Default"]
	liste_val = [calque,table]
	results = UI.inputbox prompts, val_defaut, liste_val, "Couleur"
	@calque, @colr1, @colr2, @colr3, @colr4, @colr5 = results
	calque_export if @colr1 == "Color Layer"
	
	calque = @calque	
        color1 = @colr1
        color2 = @colr1
        color3 = @colr1
        color4 = @colr1
        color5 = @colr1
        return calque, color1, color2, color3, color4, color5
    end
end

def colorEdge_params2

   def couleur
	calque=[]
	Sketchup.active_model.layers.each{|x|calque.push(x.name)}
	calque = ["All"] + calque
	calque=calque.join('|')
	table = (["Delete","Default"] + Sketchup::Color.names).join("|")
	prompts = ["图层","X轴颜色","Y轴颜色","Z轴颜色"]    
	val_defaut = ["All","Red","Green","Blue"]
	liste_val = [calque,table,table,table]
	results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
	@calque, @colr1, @colr2, @colr3, @colr4, @colr5 = results
	calque = @calque
        color1 = @colr1
        color2 = @colr2
        color3 = @colr3
        color4 = @colr1
        color5 = @colr1
        return calque, color1, color2, color3, color4, color5
    end
end

def self.coloredge_select(type_color = "single", mode_color = "base")
	model = Sketchup.active_model
	entities = model.entities
	selection = model.selection
	materials = model.materials
	layers = model.layers
	model.start_operation("ColorEdge")
	colorEdge_params1  if(type_color == "single")
        colorEdge_params  if((type_color == "multi") and(mode_color == "base"))
        colorEdge_params2  if((type_color == "multi") and(mode_color == "shade"))
        calque, color1, color2, color3, color4, color5 = couleur
	rendering_options = model.rendering_options	
	value = rendering_options["EdgeColorMode"]=0
	    if model.selection.empty?
	        oui_non = UI.messagebox "未选择，是否要将操作应用至所有实体?", MB_YESNO 
	      	if(oui_non == 6)
	       		ent = entities
	       	else 
	       		return
	       	end
	      
	    else
		ent = selection
	    end
      color1 = nil if color1 == "Delete"
      color2 = nil if color2 == "Delete"
      color3 = nil if color3 == "Delete"
      color4 = nil if color4 == "Delete"
      color5 = nil if color5 == "Delete"
      (materials.current) ? (mat_current = materials.current.color) : (mat_current = nil)
      color1 = mat_current if(color1 == "Default")
      color2 = mat_current if(color2 == "Default")
      color3 = mat_current if(color3 == "Default")
      color4 = mat_current if(color4 == "Default")
      color5 = mat_current if(color5 == "Default")
      color_x = Sketchup::Color.new color1 if color1 and color1 != "Color Layer"
      color_y = Sketchup::Color.new color2 if color2 and color1 != "Color Layer"
      color_z = Sketchup::Color.new color3 if color3 and color1 != "Color Layer"
      couleur_calque = true if color1 == "Color Layer"
 
	   ent.each {|edge|				
		if((edge.is_a? Sketchup::Edge) and ((calque == "All") or (edge.layer.name == calque)))
		    if couleur_calque
		           color1 = materials[edge.layer.name].color
		           color2 = materials[edge.layer.name].color
		           color3 = materials[edge.layer.name].color
		           color4 = materials[edge.layer.name].color
		           color5 = materials[edge.layer.name].color
		    end
		    
        		vertex0 = edge.start
        		position0 = vertex0.position
        		vertex1 = edge.end
        		position1 = vertex1.position
        		vecteur = position0.vector_to position1
        		vecteur = vecteur.normalize!
        		vecteur.x = vecteur.x.round_to(7)
        		vecteur.y = vecteur.y.round_to(7)
        		vecteur.z = vecteur.z.round_to(7)
        		vecteur.x = -vecteur.x if(vecteur.x < 0)
        		vecteur.y = -vecteur.y if(vecteur.y < 0)
        		vecteur.z = -vecteur.z if(vecteur.z < 0)
        			if((type_color == "multi") and (mode_color == "shade"))       				
        				color = color_z.blend (color_x.blend color_y, vecteur.x), vecteur.z
					edge.material = color
        			else
                			if(vecteur.x == 1)
                				edge.material = color1
                			elsif(vecteur.y == 1)
                				edge.material = color2
                			elsif(vecteur.z == 1)
                				edge.material = color3
                	    		elsif(((vecteur.x == vecteur.y) and(vecteur.x + vecteur.y != 0) ) or((vecteur.x == vecteur.z) and(vecteur.x + vecteur.z != 0) )or((vecteur.y == vecteur.z) and(vecteur.y + vecteur.z != 0) ))
                				edge.material = color4
                			else
                				edge.material = color5
                			end
                		end
		end		
		}
	selection.clear
	model.commit_operation
	model.materials.purge_unused   
end

def self.pointille_select(colr1 = "Keep", colr2 = "Invisible", diviseur = 10, unite ="%")
    
        @color = colr1
        @color1 = colr2
        @diviseur = diviseur
        @unite = unite
       
	model = Sketchup.active_model
	entities = model.entities
	selection = model.selection
	materials = model.materials
	layers = model.layers
	rendering_options = model.rendering_options	
	value = rendering_options["EdgeColorMode"]= 0
		
           unless materials["Invisible"]
            	@arriere_plan1 = rendering_options["BackgroundColor"]
            	@arriere_plan1 = @arriere_plan1.to_a
                @arriere_plan1[3] = 0
                @arriere_plan = Sketchup::Color.new @arriere_plan1
            	@background = materials.add "Invisible"
            	@background.color = @arriere_plan
           	@background = @background.name
  	   end 
   	
   	    if rendering_options["HideConstructionGeometry"] == true 
       		rendering_options["HideConstructionGeometry"] = false 
    	    end
    	    
	    if selection.empty?
	        oui_non = UI.messagebox "No selection, do you want apply\nthe operation to all entities?", MB_YESNO 
	      	if(oui_non == 6)
	       		ent = entities
	       	else 
	       		return
	       	end
	      
	    else
		ent = selection
	    end
	
	calque=[]
	layers.each{|x|calque.push(x.name)}
	calque = ["All"] + calque
	calque=calque.join('|')
        table = (["Keep","Default","Without"] + Sketchup::Color.names).join("|")
        table1 = (["Keep","Default","Invisible","Without"] + Sketchup::Color.names).join("|")
      	prompts = ["By Layer","Stippling","en % ou en cm","Couleur 1","Couleur 2"]   
      	val_defaut = ["All",10, "%", "Keep", "Invisible"]
      	liste_val = [calque,"", "%|cm", table, table1]
      	results = UI.inputbox prompts, val_defaut, liste_val, "Couleur"
      	return if not results
      	calque,diviseur, unite, color1, color2 = results
      	@calque = calque.to_s
      	@diviseur = diviseur
      	@unite = unite.to_s			
      	@color = color1.to_s
      	@color1 = color2.to_s
      		
        Sketchup.active_model.start_operation("Dotted line")
        
        edge_array = []
         ent.each {|ent|				
			if((ent.is_a? Sketchup::Edge) and ((@calque == "All") or (ent.layer.name == @calque)))
        			edge_array << ent
        		end
        	  }
        	  @color0 = "Keep" if @color == "Keep"
                  @color10 = "Keep" if @color1 == "Keep"
        	   edge_array.each {|edge|	
                            vertices_array = []
                            dep_edge = edge.start
                            fin_edge = edge.end
                            dep_edge = Geom::Point3d.new dep_edge.position
                            fin_edge = Geom::Point3d.new fin_edge.position
                            vecteur_edge = dep_edge.vector_to fin_edge
                            vertices_array << dep_edge
                            coloredge = edge.material
                            @color = materials.current if @color == "Default"
                            @color1 = materials.current if @color1 == "Default"
                            @color = nil if @color == "Without"
                            @color1 = nil if @color1 == "Without"
                            @color = coloredge if @color0 == "Keep"
                            @color1 = coloredge if @color10 == "Keep"
                                 if @unite == "cm"
                            		@nb_division =(edge.length / @diviseur.cm).ceil
                                 elsif @unite == "%"
                            		@nb_division =(100 / @diviseur).ceil
                                 end
                                 @nb_division = @nb_division -1 if @nb_division % 2 == 0
                            long_segment = edge.length / @nb_division     		
                            @nb_division.times {|x| 
                                               (x % 2 == 0) ?(color = @color) :(color = @color1)
                                                mid_edge = dep_edge.offset vecteur_edge, long_segment
                                                new_edge = entities.add_line(dep_edge, mid_edge)
                                                new_edge.material = color
                                                vertices_array << mid_edge
                                                dep_edge = mid_edge.clone
                            			}                            
                            curveEdges = entities.add_curve(vertices_array)
                            
		   }

	selection.clear
	model.commit_operation
	materials.purge_unused  
end #fin Pointille

class ColorEdge_OnClick

def initialize 	        
    @model = Sketchup.active_model
    @entities = @model.entities
    @selection = @model.selection
    @materials = @model.materials
    @rendering_options = @model.rendering_options	
    value = @rendering_options["EdgeColorMode"]=0
    @curseur1 = 0
    path = File.join(File.dirname(__FILE__),  "ChrisP_ColorEdge", "coloredge_cursor1.png")
    @curseur1 = UI::create_cursor(path, 0, 0)
end

def onSetCursor
      UI::set_cursor(@curseur1)  
end

def activate(colr1 = "Default") 
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
    @color = colr1
    
end

def deactivate(view)
    view.invalidate if @displayed
    @materials.purge_unused 
end

def onMouseMove(flags, x, y, view)
    Sketchup::set_status_text("#{x}, #{y}", SB_VCB_VALUE)
    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end

        
        if( @ip.vertex == nil )
            if( @ip.edge )
                if( @ip.depth > 0 )
                    length = @ip.edge.length(@ip.transformation)
                else
                    length = @ip.edge.length
                end
	                mat = @ip.edge.material
			if(mat)
			color = @ip.edge.material.color
			nom = @ip.edge.material.name
			else
			nom = "No color"
			color =" No RGB"
			end
                msg << "\n       " +("Couleur  :  ") + nom + "\n       " +("RGB  :  ") + color.to_s
            end
        end

        msg2 = msg.gsub(/\n/,' ')
        Sketchup::set_status_text msg2
        
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
end

def onKeyDown(key, repeat, flags, view)

	    if key == 9
              	table = (["Default","Delete"] + Sketchup::Color.names).join("|")
      		prompts = ["Color single"]   
      		val_defaut = ["Magenta"]
      		liste_val = [table]
      		results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
      		color1 = results			
      		@color = color1.to_s
	    end
	    
end
	
def onLButtonUp(flags, x, y, view)
    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end
                   
            if( @ip.edge )
                @color = nil if @color == "Delete"
               (@color == "Default") ?(@ip.edge.material = @materials.current) :(@ip.edge.material = @color)
      	     	self.reset(view)
      	    else
      	     	self.reset(view)           
            end
            
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
end

def reset(view)
	  
	    
	    Sketchup::set_status_text("TAB to modify colors", SB_PROMPT)
	    
	    @ip.clear
	    @iptemp.clear
	    if( view )
	        view.tooltip = nil
	        view.invalidate
	    end
	    
	    @displayed = false
	    
end
		
end # Fin de Class

class ColorAngle_OnClick


def initialize 	        
    @model = Sketchup.active_model
    @entities = @model.entities
    @selection = @model.selection
    @materials = @model.materials
    @rendering_options = @model.rendering_options	
    value = @rendering_options["EdgeColorMode"]=0
    
    	unless @materials["Invisible"]
          @arriere_plan1 = @rendering_options["BackgroundColor"]
          @arriere_plan1 = @arriere_plan1.to_a
          @arriere_plan1[3] = 0
          @arriere_plan = Sketchup::Color.new @arriere_plan1
          @background = @materials.add "Invisible"
          @background.color = @arriere_plan
          @background = @background.name
	end
	    
    @curseur3 = 0
    path = File.join(File.dirname(__FILE__),  "ChrisP_ColorEdge", "coloredge_cursor3.png")
    @curseur3 = UI::create_cursor(path, 0, 0)
end

def onSetCursor
      UI::set_cursor(@curseur3)  
end

def activate(colr1 = "red", colr2 = "Keep", diviseur = 5, unite ="%")
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
    @color = colr1
    @color1 = colr2
    @diviseur = diviseur
    @unite = unite

end
 
def deactivate(view)
    view.invalidate if @displayed
    @materials.purge_unused
    @curve_array.clear
end

def onMouseMove(flags, x, y, view)
    Sketchup::set_status_text("#{x}, #{y}", SB_VCB_VALUE)
    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end

        
        if( @ip.vertex == nil )
            if( @ip.edge )
                if( @ip.depth > 0 )
                    length = @ip.edge.length(@ip.transformation)
                else
                    length = @ip.edge.length
                end
	                mat = @ip.edge.material
			if(mat)
			color = @ip.edge.material.color
			nom = @ip.edge.material.name
			else
			nom = "No Color"
			color =" No RGB"
			end
                msg << "\n       " +("Color  :  ") + nom + "\n       " +("RGB  :  ") + color.to_s
            end
        end

        msg2 = msg.gsub(/\n/,' ')
        Sketchup::set_status_text msg2
        
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
end

def onKeyDown(key, repeat, flags, view)

	    if key == 9
        	table = (["Keep","Default","Without"] + Sketchup::Color.names).join("|")
       		table1 = (["Keep","Default","Invisible","Without"] + Sketchup::Color.names).join("|")
       		prompts = ["Length angle","in % ou in cm","Color Angle","Color Edge"]   
      		val_defaut = [5, "%", "Red", "Keep"]
      		liste_val = ["", "%|cm", table, table1]
      		results = UI.inputbox prompts, val_defaut, liste_val, "Color Angle"
      		return if not results
      		diviseur, unite, color1, color2 = results
      		@diviseur = diviseur
      		@unite = unite.to_s			
      		@color = color1.to_s
      		@color1 = color2.to_s
      		
	    end
	    
end
	
def onLButtonUp(flags, x, y, view)

    @model.start_operation("Color Angle")
    @curve_array = [] if @curve_array == nil
    @iptemp.pick view, x, y
    if( @iptemp.valid? )    
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position        
        msg = @ip.tooltip       
        if( msg.length > 0 )
            msg << " "
        end
                   
            if( @ip.edge )
                vertices_array = []
                dep_edge = @ip.edge.start
                fin_edge = @ip.edge.end
                dep_edge = Geom::Point3d.new dep_edge.position
                fin_edge = Geom::Point3d.new fin_edge.position
                vecteur_edge = dep_edge.vector_to fin_edge
                vertices_array << dep_edge
                @coloredge = @ip.edge.material
                @color0 = @color
                @color10 = @color1
                @color0 = @materials.current if @color == "Default"
      		@color10 = @materials.current if @color1 == "Default"
		@color0 = nil if @color == "Without"
      		@color10 = nil if @color1 == "Without"
      		@color0 = @coloredge if @color == "Keep"
      		@color10 = @coloredge if @color1 == "Keep"
      		     if @unite == "cm"
      			@long_angle = @diviseur.cm
      			@long_reste = @ip.edge.length - (@long_angle * 2)
      		     elsif @unite == "%"
      			@long_angle = @ip.edge.length * (@diviseur.to_f / 100)
      			@long_reste = @ip.edge.length - (@long_angle * 2)
      		     end
      	       mid_edge = dep_edge.offset vecteur_edge, @long_angle
               new_edge = @entities.add_line(dep_edge, mid_edge)
               new_edge.material = @color0
               vertices_array << mid_edge
               dep_edge = mid_edge.clone
                     
      	       mid_edge = dep_edge.offset vecteur_edge, @long_reste
               new_edge = @entities.add_line(dep_edge, mid_edge)
               new_edge.material = @color10
               vertices_array << mid_edge
               dep_edge = mid_edge.clone
               
               mid_edge = dep_edge.offset vecteur_edge, @long_angle
               new_edge = @entities.add_line(dep_edge, mid_edge)
               new_edge.material = @color0
               vertices_array << mid_edge
               dep_edge = mid_edge.clone     
      		     
               curveEdges = @entities.add_curve(vertices_array)
               @selection.clear               
      	       self.reset(view)
      	    elsif( @ip.face )
      	         edges = @ip.face.edges      	         
      	         edges.each {|edge|
      	            if not(@curve_array.include?(edge))
      	                vertices_array = []
	                dep_edge = edge.start
	                fin_edge = edge.end
	                dep_edge = Geom::Point3d.new dep_edge.position
	                fin_edge = Geom::Point3d.new fin_edge.position
	                vecteur_edge = dep_edge.vector_to fin_edge
	                vertices_array << dep_edge
	                @coloredge = edge.material
	                @color0 = @color
	                @color10 = @color1
	                @color0 = @materials.current if @color == "Default"
	      		@color10 = @materials.current if @color1 == "Default"
			@color0 = nil if @color == "Without"
	      		@color10 = nil if @color1 == "Without"
	      		@color0 = @coloredge if @color == "Keep"
	      		@color10 = @coloredge if @color1 == "Keep"
	      		     if @unite == "cm"
	      			@long_angle = @diviseur.cm
	      			@long_reste = edge.length - (@long_angle * 2)
	      		     elsif @unite == "%"
      				@long_angle = edge.length * (@diviseur.to_f / 100)
	      			@long_reste = edge.length - (@long_angle * 2)
	      		     end
	      	       mid_edge = dep_edge.offset vecteur_edge, @long_angle
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array << new_edge
	               new_edge.material = @color0
	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone
	                     
	      	       mid_edge = dep_edge.offset vecteur_edge, @long_reste
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array << new_edge
	               new_edge.material = @color10
	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone
	               
	               mid_edge = dep_edge.offset vecteur_edge, @long_angle
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array << new_edge
	               new_edge.material = @color0
	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone     
	      		     
	               curveEdges = @entities.add_curve(vertices_array)
	               @selection.clear               
	      	       self.reset(view)
	      	    end
	      	 }
	      	    
      	    else
      	     	self.reset(view)           
            end
            
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
    @model.commit_operation
end

def onLButtonDoubleClick(flags, x, y, view)

    @model.start_operation("Color Angle")
    @curve_array1 = [] if @curve_array1 == nil    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end
                   
             if( (@ip.face) or (@ip.edge) )
             	(@ip.face) ? (edges = @ip.face.all_connected) : (edges = @ip.edge.all_connected)             	
             	@ip.face.edges.each {|ed| @curve_array1 << ed } if(@ip.face)
             	@curve_array1 << @ip.edge if(@ip.edge)
             	edge_array = []
             	edges.each {|edge|
             		      if(edge.is_a? Sketchup::Edge)
             		    	edge_array << edge
             		      end
             	            }
             	                
      	         edge_array.each {|edge|
      	            if not(@curve_array1.include?(edge))
      	                vertices_array = []
	                dep_edge = edge.start
	                fin_edge = edge.end
	                dep_edge = Geom::Point3d.new dep_edge.position
	                fin_edge = Geom::Point3d.new fin_edge.position
	                vecteur_edge = dep_edge.vector_to fin_edge
	                vertices_array << dep_edge
	                @coloredge = edge.material
	                @color0 = @color
	                @color10 = @color1
	                @color0 = @materials.current if @color == "Default"
	      		@color10 = @materials.current if @color1 == "Default"
			@color0 = nil if @color == "Without"
	      		@color10 = nil if @color1 == "Without"
	      		@color0 = @coloredge if @color == "Keep"
	      		@color10 = @coloredge if @color1 == "Keep"
	      		     if @unite == "cm"
	      			@long_angle = @diviseur.cm
	      			@long_reste = edge.length - (@long_angle * 2)
	      		     elsif @unite == "%"
      				@long_angle = edge.length * (@diviseur.to_f / 100)
	      			@long_reste = edge.length - (@long_angle * 2)
	      		     end
	      	       mid_edge = dep_edge.offset vecteur_edge, @long_angle
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array1 << new_edge
	               new_edge.material = @color0
	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone
	                     
	      	       mid_edge = dep_edge.offset vecteur_edge, @long_reste
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array1 << new_edge
	               new_edge.material = @color10
	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone
	               
	               mid_edge = dep_edge.offset vecteur_edge, @long_angle
	               new_edge = @entities.add_line(dep_edge, mid_edge)
	               @curve_array1 << new_edge
	               new_edge.material = @color0
 	               vertices_array << mid_edge
	               dep_edge = mid_edge.clone     
	      		     
	               curveEdges = @entities.add_curve(vertices_array)
	               @selection.clear               
	      	       self.reset(view)
	      	   end  
	      	 }
	      	    
      	    else
      	     	self.reset(view)           
            end
            
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
    @model.commit_operation
end

def reset(view)
	  
	    
	    Sketchup::set_status_text("TAB to modify Parameters", SB_PROMPT)
	    
	    @ip.clear
	    @iptemp.clear
	    if( view )
	        view.tooltip = nil
	        view.invalidate
	    end
	    
	    @displayed = false
end
		
end # Fin de Class

class Pointille_OnClick

def initialize 	        
    @model = Sketchup.active_model
    @entities = @model.entities
    @selection = @model.selection
    @materials = @model.materials
    @rendering_options = @model.rendering_options	
    value = @rendering_options["EdgeColorMode"]=0
    
    	unless @materials["Invisible"]
          @arriere_plan1 = @rendering_options["BackgroundColor"]
          @arriere_plan1 = @arriere_plan1.to_a
          @arriere_plan1[3] = 0
          @arriere_plan = Sketchup::Color.new @arriere_plan1
          @background = @materials.add "Invisible"
          @background.color = @arriere_plan
          @background = @background.name
	end
	    
    @curseur1 = 0
    path = File.join(File.dirname(__FILE__),  "ChrisP_ColorEdge", "coloredge_cursor2.png")
    @curseur2 = UI::create_cursor(path, 0, 0)
end

def onSetCursor
      UI::set_cursor(@curseur2)  
end

def activate(colr1 = "Keep", colr2 = "Invisible", diviseur = 10, unite ="%")
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
    @color = colr1
    @color1 = colr2
    @diviseur = diviseur
    @unite = unite
 
end
 
def deactivate(view)
    view.invalidate if @displayed
    @materials.purge_unused 
end

def onMouseMove(flags, x, y, view)
    Sketchup::set_status_text("#{x}, #{y}", SB_VCB_VALUE)
    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end

        
        if( @ip.vertex == nil )
            if( @ip.edge )
                if( @ip.depth > 0 )
                    length = @ip.edge.length(@ip.transformation)
                else
                    length = @ip.edge.length
                end
	                mat = @ip.edge.material
			if(mat)
			color = @ip.edge.material.color
			nom = @ip.edge.material.name
			else
			nom = "No color"
			color ="No RGB"
			end
                msg << "\n       " +("Color  :  ") + nom + "\n       " +("RGB  :  ") + color.to_s
            end
        end

        msg2 = msg.gsub(/\n/,' ')
        Sketchup::set_status_text msg2
        
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
end

def onKeyDown(key, repeat, flags, view)

	    if key == 9
              	table = (["Keep","Default","Without"] + Sketchup::Color.names).join("|")
       		table1 = (["Keep","Default","Invisible","Without"] + Sketchup::Color.names).join("|")
       		prompts = ["Stippling","in % ou in cm","Color 1","Color 2"]   
      		val_defaut = [10, "%", "Keep", "Invisible"]
      		liste_val = ["", "%|cm", table, table1]
      		results = UI.inputbox prompts, val_defaut, liste_val, "Colors"
      		return if not results
      		diviseur, unite, color1, color2 = results
      		@diviseur = diviseur
      		@unite = unite.to_s			
      		@color = color1.to_s
      		@color1 = color2.to_s
      		
	    end
	    
end
	
def onLButtonUp(flags, x, y, view)

    @model.start_operation("Dotted line")
    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position
        
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end
                   
            if( @ip.edge )
                vertices_array = []
                dep_edge = @ip.edge.start
                fin_edge = @ip.edge.end
                dep_edge = Geom::Point3d.new dep_edge.position
                fin_edge = Geom::Point3d.new fin_edge.position
                vecteur_edge = dep_edge.vector_to fin_edge
                vertices_array << dep_edge
                @coloredge = @ip.edge.material
                @color0 = @color
                @color10 = @color1
                @color0 = @materials.current if @color == "Default"
      		@color10 = @materials.current if @color1 == "Default"
		@color0 = nil if @color == "Without"
      		@color10 = nil if @color1 == "Without"
      		@color0 = @coloredge if @color == "Keep"
      		@color10 = @coloredge if @color1 == "Keep"
      		     if @unite == "cm"
      			@nb_division =(@ip.edge.length / @diviseur.cm).ceil
      		     elsif @unite == "%"
      			@nb_division =(100 / @diviseur).ceil
      		     end
      		     @nb_division = @nb_division -1 if @nb_division % 2 == 0
      		long_segment = @ip.edge.length / @nb_division
                @nb_division.times {|x| 
                (x % 2 == 0) ?(color = @color0) :(color = @color10)
                mid_edge = dep_edge.offset vecteur_edge, long_segment
                new_edge = @entities.add_line(dep_edge, mid_edge)
                new_edge.material = color
                vertices_array << mid_edge
                dep_edge = mid_edge.clone
                }
                curveEdges = @entities.add_curve(vertices_array)
                @selection.clear               
      	     	self.reset(view)
      	    else
      	     	self.reset(view)           
            end
            
        view.tooltip = msg
        
        if( changed and(@ip.display? or @displayed) )
            view.invalidate
        end
    end
    
    @model.commit_operation
end

def reset(view)
	  
	    
	    Sketchup::set_status_text("TAB to modify colors", SB_PROMPT)
	    
	    @ip.clear
	    @iptemp.clear
	    if( view )
	        view.tooltip = nil
	        view.invalidate
	    end
	    
	    @displayed = false
end
		
end # Fin de Class

def self.nuancier
        my_dialog = UI::WebDialog.new("Color html", false, "Color Html", 400, 300, 400, 400, true)
        html_path = Sketchup.find_support_file "color.html" ,"Plugins/ChrisP_ColorEdge"
        my_dialog.set_file(html_path)
        my_dialog.show()
end

unless file_loaded?("ChrisP_ColorEdge.rb")

  toolbar = UI::Toolbar.new("彩线插件-云中库汉化")
  
  cmd0 = UI::Command.new("彩线插件-云中库汉化")  { Sketchup.active_model.select_tool  ColorEdge.new("single") }
  cmd0.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "couleurunique_icon_sm.png")
  cmd0.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "couleurunique_icon_lg.png")
  cmd0.tooltip = "根据当前选择的材质颜色画彩线"
  cmd0.menu_text = "根据当前选择的材质颜色画彩线"
  toolbar.add_item(cmd0)
  #UI.menu("Draw").add_item(cmd0)
  
  cmd1 = UI::Command.new("彩线插件-云中库汉化")  { Sketchup.active_model.select_tool  ColorEdge.new("multi") }
  cmd1.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "cinqcouleurs_icon_sm.png")
  cmd1.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "cinqcouleurs_icon_lg.png")
  cmd1.tooltip = "按方向确定绘制线的颜色"
  cmd1.menu_text = "按方向确定绘制线的颜色"
  toolbar.add_item(cmd1)
  #UI.menu("Draw").add_item(cmd1)
  
  cmd2 = UI::Command.new("彩线插件-云中库汉化")  { self.coloredge_select("multi","base") }
  cmd2.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_icon_sm.png")
  cmd2.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_icon_lg.png")
  cmd2.tooltip = "在选择或模型上按轴设置边缘颜色"
  cmd2.menu_text = "在选择或模型上按轴设置边缘颜色"
  toolbar .add_item(cmd2)
  #UI.menu("Draw").add_item(cmd2)
  
  cmd3 = UI::Command.new("彩线插件-云中库汉化")  { self.coloredge_select("single","base") }
  cmd3.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_unique_sm.png")
  cmd3.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_unique_lg.png")
  cmd3.tooltip = "在选择或模型上设置单色边缘"
  cmd3.menu_text = "在选择或模型上设置单色边缘"
  toolbar .add_item(cmd3)
  #UI.menu("Draw").add_item(cmd3)
  
  cmd5 = UI::Command.new("彩线插件-云中库汉化")  { self.coloredge_select("multi","shade") }
  cmd5.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "degrade_icon_sm.png")
  cmd5.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "degrade_icon_lg.png")
  cmd5.tooltip = "边线的颜色根据线的方向确定（3个轴线颜色的中间色）"
  cmd5.menu_text = "边线的颜色根据线方向确定（3个轴线颜色的中间色）"
  toolbar .add_item(cmd5)
  #UI.menu("Draw").add_item(cmd3)
  
  cmd6 = UI::Command.new("彩线插件-云中库汉化")  { Sketchup.active_model.select_tool  ColorEdge_OnClick.new }
  cmd6.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "ColorEdge_OnClick_sm.png")
  cmd6.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "ColorEdge_OnClick_lg.png")
  cmd6.tooltip = "通过点击设置边缘颜色"
  cmd6.menu_text = "通过点击设置边缘颜色"
  toolbar.add_item(cmd6)
  #UI.menu("Draw").add_item(cmd6)
  
  cmd7 = UI::Command.new("彩线插件-云中库汉化")  { Sketchup.active_model.select_tool  Pointille_OnClick.new }
  cmd7.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "Pointille_OnClick_sm.png")
  cmd7.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "Pointille_OnClick_lg.png")
  cmd7.tooltip = "实线变虚线"
  cmd7.menu_text = "实线变虚线"
  toolbar.add_item(cmd7)
  #UI.menu("Draw").add_item(cmd6)
  
  cmd8 = UI::Command.new("彩线插件-云中库汉化")  { self.pointille_select("Keep", "Invisible", 10, "%") }
  cmd8.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_pointille_sm.png")
  cmd8.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "select_pointille_lg.png")
  cmd8.tooltip = "把选中的线改为虚线"
  cmd8.menu_text = "把选中的线改为虚线"
  toolbar .add_item(cmd8)
  #UI.menu("Draw").add_item(cmd8)

  cmd9 = UI::Command.new("Aretes colorisees")  { Sketchup.active_model.select_tool  ColorAngle_OnClick.new }
  cmd9.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "Angle_OnClick_sm.png")
  cmd9.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "Angle_OnClick_lg.png")
  cmd9.tooltip = "查看彩线的RGB值"
  cmd9.menu_text = "查看彩线的RGB值"
  toolbar.add_item(cmd9)
  #UI.menu("Draw").add_item(cmd9)
      
  cmd4 = UI::Command.new("彩线插件-云中库汉化")  { self.nuancier }
  cmd4.small_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "nuancier_sm.png")
  cmd4.large_icon = File.join(File.dirname(__FILE__), "ChrisP_ColorEdge", "nuancier_lg.png")
  cmd4.tooltip = "颜色列表（调色盘）"
  cmd4.menu_text = "颜色列表（调色盘"
  toolbar .add_item(cmd4)
  #UI.menu("Draw").add_item(cmd4)

  file_loaded "ChrisP_ColorEdge.rb"
end
